/*
 * #%L
 * EUGene :: EUGene
 * 
 * $Id: JavaGeneratorUtil.java 910 2010-05-27 16:37:38Z fdesbois $
 * $HeadURL: http://svn.nuiton.org/svn/eugene/tags/eugene-2.1.2/eugene/src/main/java/org/nuiton/eugene/java/JavaGeneratorUtil.java $
 * %%
 * Copyright (C) 2004 - 2010 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
package org.nuiton.eugene.java;

import org.apache.commons.lang.StringUtils;
import org.nuiton.eugene.GeneratorUtil;
import org.nuiton.eugene.models.object.ObjectModelAttribute;
import org.nuiton.eugene.models.object.ObjectModelClass;
import org.nuiton.eugene.models.object.ObjectModelClassifier;
import org.nuiton.eugene.models.object.ObjectModelModifier;
import org.nuiton.eugene.models.object.ObjectModelOperation;
import org.nuiton.eugene.models.object.ObjectModelParameter;

import java.util.Collection;
import java.util.List;

/**
 * Utility class for pure java templates.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 2.0.2
 */
public class JavaGeneratorUtil extends GeneratorUtil {

    public static final String OPERATION_GETTER_DEFAULT_PREFIX = "get";

    public static final String OPERATION_GETTER_BOOLEAN_PREFIX = "is";

    /** Stereotype for JavaBean objects. */
    public static final String STEREOTYPE_BEAN = "bean";

    /**
     * tag value to add on constants enumeration (or other incoming dev)
     * a prefix to constant to generate.
     */
    public static final String TAG_CONSTANT_PREFIX = "constantPrefix";
    /**
     * dependency to add constants in interfaces via an enumeration.
     *
     * The literals of enumeration will be the value, and constant names will
     * be generated by transformers.
     * @since 2.0.2
     */
    public static final String DEPENDENCIES_CONSTANTS = "constants";

    /**
     * Duplicates in the {@code target} classifier the given {@code operations}
     * using a {@code transformer} to modify model.
     *
     * @param transformer   the transformer to use
     * @param operations    operations to duplicate
     * @param target        where to duplicate operations
     * @param useVisibility flag to use operation visibilty to compute his
     *                      modifiers
     * @param extraModifiers        scopes to apply to all operations
     */
    public static void cloneOperations(ObjectModelTransformerToJava transformer,
                                      Iterable<ObjectModelOperation> operations,
                                      ObjectModelClassifier target,
                                      boolean useVisibility,
                                      ObjectModelModifier... extraModifiers) {

        for (ObjectModelOperation op : operations) {
            ObjectModelOperation resultOp;
            ObjectModelModifier[] modifiers = null;
            if (useVisibility) {

                // compute visibility modifer
                String visibility = op.getVisibility();
                ObjectModelModifier modifier =
                        ObjectModelModifier.toValue(visibility);

                int length = extraModifiers.length;
                if (length == 0) {
                    modifiers = new ObjectModelModifier[]{modifier};
                } else {
                    modifiers = new ObjectModelModifier[length + 1];
                    modifiers[0] = modifier;
                    System.arraycopy(extraModifiers, 0, modifiers, 1, length);
                }
            } else {

                // just use the incoming modifiers
                modifiers = extraModifiers;
            }

            resultOp = transformer.addOperation(target,
                                                op.getName(),
                                                op.getReturnType(),
                                                modifiers
            );
            for (ObjectModelParameter param : op.getParameters()) {
                transformer.addParameter(resultOp,
                                         param.getType(),
                                         param.getName()
                );
            }
            for (String exception : op.getExceptions()) {
                transformer.addException(resultOp, exception);
            }
        }
    }
}
